package
{
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
    import flash.utils.Dictionary;

    public class PhysicsData extends Object
	{
		// ptm ratio
        public var ptm_ratio:Number = {{global.ptm_ratio}};
		
		// the physcis data 
		var dict:Dictionary;
		
        //
        // bodytype:
        //  b2_staticBody
        //  b2_kinematicBody
        //  b2_dynamicBody

        public function createBody(name:String, world:b2World, bodyType:uint, userData:*):b2Body
        {
            var fixtures:Array = dict[name];

            var body:b2Body;
            var f:Number;

            // prepare body def
            var bodyDef:b2BodyDef = new b2BodyDef();
            bodyDef.type = bodyType;
            bodyDef.userData = userData;

            // create the body
            body = world.CreateBody(bodyDef);

            // prepare fixtures
            for(f=0; f<fixtures.length; f++)
            {
                var fixture:Array = fixtures[f];

                var fixtureDef:b2FixtureDef = new b2FixtureDef();

                fixtureDef.density=fixture[0];
                fixtureDef.friction=fixture[1];
                fixtureDef.restitution=fixture[2];

                fixtureDef.filter.categoryBits = fixture[3];
                fixtureDef.filter.maskBits = fixture[4];
                fixtureDef.filter.groupIndex = fixture[5];
                fixtureDef.isSensor = fixture[6];

                if(fixture[7] == "POLYGON")
                {                    
                    var p:Number;
                    var polygons:Array = fixture[8];
                    for(p=0; p<polygons.length; p++)
                    {
                        var polygonShape:b2PolygonShape = new b2PolygonShape();
                        polygonShape.SetAsArray(polygons[p], polygons[p].length);
                        fixtureDef.shape=polygonShape;

                        body.CreateFixture(fixtureDef);
                    }
                }
                else if(fixture[7] == "CIRCLE")
                {
                    var circleShape:b2CircleShape = new b2CircleShape(fixture[9]);                    
                    circleShape.SetLocalPosition(fixture[8]);
                    fixtureDef.shape=circleShape;
                    body.CreateFixture(fixtureDef);                    
                }                
            }

            return body;
        }

		
        public function PhysicsData(): void
		{
			dict = new Dictionary();
			
{% for body in bodies %}
			dict["{{body.name}}"] = [
{% for fixture in body.fixtures %}{% if not forloop.first %} ,{% endif %}
										[
											// density, friction, restitution
                                            {{fixture.density}}, {{fixture.friction}}, {{fixture.restitution}},
                                            // categoryBits, maskBits, groupIndex, isSensor
											{{fixture.filter_categoryBits}}, {{fixture.filter_maskBits}}, {{fixture.filter_groupIndex}}, {% if fixture.isSensor %}true{% else %}false{% endif %},
											'{{fixture.type}}',
{% if fixture.isCircle %}
                                            // center, radius
                                            new b2Vec2({{fixture.center.x|floatformat:3}}/ptm_ratio,{{fixture.center.y|floatformat:3}}/ptm_ratio),
                                            {{fixture.radius|floatformat:3}}/ptm_ratio
{% else %}
                                            // vertexes of decomposed polygons
                                            [
{% for polygon in fixture.polygons %}{% if not forloop.first %} ,{% endif %}
                                                [ {% for point in polygon %} {% if not forloop.first %}, {% endif %} new b2Vec2({{point.x}}/ptm_ratio, {{point.y}}/ptm_ratio) {% endfor %} ]{% endfor %}
											]
{% endif %}
										]
{% endfor %}
									];
{% endfor %}
		}
	}
}
